//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                Contains Gaussian Blur               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Initial Gaussian Blur code                  //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

    int BLUR_QUALITY <
      string UIName = "Choose Blur Quality";  string UIWidget="Quality";
    > = {2};



// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 GaussBlur(float4 inColor, float4 inBloom, float2 inCoord)
{
  inBloom            = inColor;
  float sHeight      = ScreenSize.x * ScreenSize.w;
  float py           = 1.0 / sHeight;
  float px           = ScreenSize.y;
  float SigmaSum     = 0.0f;
  float sampleOffset = 1.0f;


// Quality
  float fBlurSigma;
  float Q;
if (BLUR_QUALITY==2) {
  Q=7;
  fBlurSigma=3.5;
}
else if (BLUR_QUALITY==1) {
  Q=11;
  fBlurSigma=5.5;
}
else if (BLUR_QUALITY==0) {
  Q=15;
  fBlurSigma=7.5;
}
else if (BLUR_QUALITY==-1) {
  Q=19;
  fBlurSigma=9.5;
}

/// Gaussian
    float3 Sigma;
      Sigma.x = 1.0f / ( sqrt( 2.0f * PI ) * fBlurSigma );
      Sigma.y = exp( -0.5f / ( fBlurSigma * fBlurSigma ));
      Sigma.z = Sigma.y * Sigma.y;

/// Center weight
    inBloom  *= Sigma.x;
    SigmaSum += Sigma.x;
    Sigma.xy *= Sigma.yz;

    for(int i = 0; i < Q; ++i) {
      inBloom     += TextureAdaptation.Sample(Sampler0, inCoord.xy + float2(sampleOffset * px, 0.0)) * Sigma.x;
      inBloom     += TextureAdaptation.Sample(Sampler0, inCoord.xy - float2(sampleOffset * px, 0.0)) * Sigma.x;
      SigmaSum    += (2.0f * Sigma.x);
      sampleOffset = sampleOffset + 1.0f;
      Sigma.xy    *= Sigma.yz;
    }
  inBloom.xyz /= SigmaSum;

    for(int i = 0; i < Q; ++i) {
      inBloom     += TextureAdaptation.Sample(Sampler0, inCoord.xy + float2(0.0, sampleOffset * py)) * Sigma.x;
      inBloom     += TextureAdaptation.Sample(Sampler0, inCoord.xy - float2(0.0, sampleOffset * py)) * Sigma.x;
      SigmaSum    += ( 2.0f * Sigma.x );
      sampleOffset = sampleOffset + 1.0f;
      Sigma.xy    *= Sigma.yz;
    }
  inBloom.xyz /= SigmaSum;

  return inBloom;
}
