//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains Adaptation                 //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Author of Adaptation Curve                  //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - DNSrSsEIFactor(WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight)
 * - DNEIFactor(NightExt, DayExt, NightInt, DayInt)                       *
 * - DNIFactor(NightExt, DayExt, Int)                                     *
 * - DNSrSsEIFactor(_c3, WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight) // ELE only!
 * - DNIDFactor(_c3, NightExt, DayExt, Interior, Dungeon) // ELE only!    *
 * - IDFactor(_c3, Interior, Dungeon)                     // ELE only!    *
 * - DunFactor(_c3, NightDayInt, Dungeon)                 // ELE only!    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Row_AdaptDepth <
  string UIName="-----------------------ADAPTATION";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

//{
  bool ENABLE_ADAPTDEPTH <
    string UIName = "Enable Adaptation Depth Control";
  > = {true};
    int fAdaptDepth <
      string UIName="ADAPTATION: Depth";  string UIWidget="Spinner";  int UIMin=0.0;  int UIMax=100000.0;
    > = {1000.0};
//}

// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msAdaptation(float3 inAdapt, float2 inCoord)
{
    float Depth    = TextureDepth.Sample(Sampler0, inCoord.xy ).x;
    float linDepth = linearDepth(Depth, 0.5f, fAdaptDepth);
    if (ENABLE_ADAPTDEPTH==true)  inAdapt.xyz = inAdapt.xyz * (1.0f - linDepth);

  return inAdapt;
}