//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains Adaptation                 //
//          used by the Modular Shader files           //
//-----------------------CREDITS-----------------------//
// Matso: Author of Adaptation tint, letter box        //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - DNSrSsEIFactor(WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight)
 * - DNEIFactor(NightExt, DayExt, NightInt, DayInt)                       *
 * - DNIFactor(NightExt, DayExt, Int)                                     *
 * - DNSrSsEIFactor(_c3, WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight) // ELE only!
 * - DNIDFactor(_c3, NightExt, DayExt, Interior, Dungeon) // ELE only!    *
 * - IDFactor(_c3, Interior, Dungeon)                     // ELE only!    *
 * - DunFactor(_c3, NightDayInt, Dungeon)                 // ELE only!    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

//{
    float fAdaptationTintAmount <
        string UIName="ADAPTATION TINT: Adaptation tint amount";string UIWidget="Spinner";float UIMin=0.0;float UIMax=0.5;
    > = {0.05};
    float fAdaptationTintBias <
        string UIName="ADAPTATION TINT: Adaptation tint bias";string UIWidget="Spinner";float UIMin=0.02;float UIMax=1.0;
    > = {0.2};
    float fAdaptationTintSaturation <
        string UIName="ADAPTATION TINT: Adaptation tint saturation";string UIWidget="Spinner";float UIMin=0.0;float UIMax=10.0;
    > = {1.0};
//}

string LETTER_BOX = "---------------------LETTER BOX";
    float fLetterboxOffset <
        string UIName="LETTER BOX: Letter box size, in per cent";string UIWidget="Spinner";float UIMin=0.0;float UIMax=50.0;
    > = {0.0};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //

// Letterbox effect parameters. [TODO: Use ENB's internal parameter providing inverted screen sizes.]
#define fScreenWidth   1920.0    // set your resolution sizes
#define fScreenHeight  1080.0
float2 fvTexelSize = float2(1.0 / fScreenWidth, 1.0 / fScreenHeight);


// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

// Returns a color tint sampled according to adaptation level given.
float3 GetAdaptationTint(float3 inColor, float3 inAdapt)
{
    float adaptation = max(max(inAdapt.r, inAdapt.g), inAdapt.b);

    float grayColor = dot(inColor, float3(0.3, 0.59, 0.11));
    float blendFactor = (1.0 - adaptation) * (grayColor + fAdaptationTintBias) * fAdaptationTintAmount;
    float tempGray = dot(inAdapt, 0.3333);
    float3 tint = lerp(tempGray, inAdapt, fAdaptationTintSaturation);
  
  return tint * blendFactor;
}

// Maps given color at the given texture position to be "letter-boxed-out" (set to black).
// Any pixels are above or below (screen vertical axis wise, Y) the threshold will be blacked out.
float3 MapLetterBox(float3 inColor, float2 texPos)
{
    float2 sspos = fvTexelSize * texPos;// [TODO: Use ENB's internal parameter providing inverted screen sizes.]

    if (sspos.y <= (fLetterboxOffset * 0.01) || sspos.y >= (1.0 - fLetterboxOffset * 0.01))
      inColor = float3(0.0, 0.0, 0.0);

  return inColor;
}
