//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                Contains Bloom Process               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Author of Bloom Process                     //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //


//{
    float fSampleRange <
        string UIName="BLOOM: Sample Range";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1.0;
    > = {0.5};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msBloomProcess(float3 inBloom, float2 inCoord)
{
  float2 invSize = ScreenSize.y;
    invSize.y   *= ScreenSize.z;
  float2 txSize  = 1 / invSize.xy;
  float2 bcoord  = inCoord.xy * txSize.xy;
    bcoord       = floor( bcoord.xy ) + 0.5f;
    bcoord      *= invSize.xy;
  float2 steps   = txSize.xy / 512;
    steps        = floor( steps + 0.5f );
    steps        = max( steps, 2 );
    steps       *= invSize.xy;
    inBloom.xyz += TextureBloom.Sample(Sampler1, float2( -fSampleRange, fSampleRange ) * steps.xy + bcoord.xy );
    inBloom.xyz += TextureBloom.Sample(Sampler1, float2( -fSampleRange, -fSampleRange ) * steps.xy + bcoord.xy );
    inBloom.xyz += TextureBloom.Sample(Sampler1, float2( fSampleRange, -fSampleRange ) * steps.xy + bcoord.xy );
    inBloom.xyz += TextureBloom.Sample(Sampler1, float2( fSampleRange, fSampleRange ) * steps.xy + bcoord.xy );
    inBloom.xyz *= 0.25;

  return inBloom;
}