//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                Contains Bloom Control               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// soulwynd: Author of Bloom Control                   //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

//{
    float fLightSpreadThreshold <
      string UIName="BLOOM: Light Spread Threshold";    string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0};
    float fLightSpreadPower <
      string UIName="BLOOM: Light Spread Power";        string UIWidget="Spinner";  float UIMin=0.0;
    > = {0.0};
    float fGlowThreshold <
      string UIName="BLOOM: Glow Threshold";            string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0};
    float fGlowPower <
      string UIName="BLOOM: Glow Power";                string UIWidget="Spinner";  float UIMin=0.0;
    > = {0.0};
    float fBloomMaxWhite <
        string UIName="BLOOM: Max White";               string UIWidget="Spinner";  float UIMin=0.01;  float UIMax=25.0;
    > = {1.0};
    float fBSaturation <
        string UIName="BLOOM: Saturation";              string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;
    > = {1.0};
    float3 fBLumRGB <
        string UIName="BLOOM: RGB Channel Filter";      string UIWidget="Color";
    > = {1.0, 1.0, 1.0};
//}



// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msBloomControl(float3 inBloom, float2 inCoord)
{

/// Bloom Lightspread
    inBloom.xyz += max(0.0, (inBloom.xyz - fLightSpreadThreshold * 0.1) * fLightSpreadPower * (1.0 - TextureBloom.Sample(Sampler1, inCoord.xy)));

/// Bloom Glow
    inBloom.xyz += max(0.0, (inBloom.xyz - fGlowThreshold * 0.1) * fGlowPower);

/// Compress and scale bloom
    inBloom = (inBloom * (1.0f + inBloom / (fBloomMaxWhite * fBloomMaxWhite))) / (1.0f + inBloom); /// Scale bloom luminance within a displayable range of 0 to 1

/// Luminance Saturation
    float3 avgBloomLuma = AvgLuma(inBloom.xyz).w;  /// Average the scene bloom luminance
    inBloom.xyz = lerp(avgBloomLuma.xyz, inBloom.xyz, fBSaturation * float3(fBLumRGB.x, fBLumRGB.y, fBLumRGB.z));

  return inBloom;
}