//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//            Contains Game Post-Processing            //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// Boris: For ENBSeries and his knowledge and codes    //
// JawZ: Author and developer of this file             //
// kingeric1992: For the initial code                  //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the elepHelpers.fxh is loaded/included into the enbeffect.fx file!

/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

//{
    float fShoulderStrength <
        string UIName="AGCC: Shoulder Strength";  string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=2.0;
    > = {1.1};
    float fLinearStrength <
        string UIName="AGCC: Linear Strength";    string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;
    > = {0.45};
    float fLinearAngle <
        string UIName="AGCC: Linear Angle";       string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=1.0;
    > = {0.16};
    float fToeStrength <
        string UIName="AGCC: Toe Strength";       string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=2.0;
    > = {0.08};
//    float fToeNumerator <
//        string UIName="AGCC: Toe Numerator";      string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=0.5;  float UIStep=0.001;
//    > = {0.001};
    float fToeDenominator <
        string UIName="AGCC: Toe Denominator";    string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=2.0;
    > = {0.3};
    float fLinearWhite <
        string UIName="AGCC: Linear White";       string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {3.33};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 Tonemap_Filmic(float3 color, float W, float A, float B, float C, float D, float E, float F)
{
    float4 res = float4(color.rgb, W);
    res        = (res * ( A * res + C * B) + D * E) / (res * ( A * res + B) + D * F);
    res       -= E / F;
    return res.rgb / res.a;
}


float4 msAGCC(float4 inColor, float2 inCoords)
{
// Fallout4 vanilla post process. Just an example for modders, better not enable without know how to edit and what you need
    float2 bloom_offset = Params01[4].zw;   /// bloom coord scaling
    float3 adaptation   = Params01[1].xyz;  /// .x = adaption max, .y = min, .z = scale/sensitivity

    float3 bloom        = TextureBloom.Sample(Sampler1, bloom_offset * inCoords.xy).rgb * ENBParams01.x;
      bloom             = msBloomDepth(inColor, bloom, inCoords.xy);
    float  middlegray   = TextureAdaptation.Sample(Sampler0, inCoords.xy).x;
      middlegray        = msAdaptation(middlegray, inCoords.xy);
    
    float  adapt = clamp(adaptation.y, adaptation.x, adaptation.z / (0.001 + middlegray));
    inColor.rgb  = (inColor.rgb + bloom.rgb) * adapt;    /// bloom in add mode

// Filmic operator
    float A = fShoulderStrength;
    float B = fLinearStrength;
    float C = fLinearAngle;
    float D = fToeStrength;
    float E = Params01[1].w;      /// Toe Numerator
    float F = fToeDenominator;
    float W = fLinearWhite;

    inColor.rgb = Tonemap_Filmic(inColor.rgb, W, A, B, C, D, E, F);

// Color corrections
    float  saturation  = Params01[2].x;   /// 0 == gray scale
    float4 tint        = Params01[3];     /// tint color
    float  contrast    = Params01[2].z;   /// 0 == no contrast
    float  brightness  = Params01[2].w;   /// intensity
    float4 fade        = Params01[5];     /// fade current scene to specified color, mostly used in special effects

    inColor.a   = dot(inColor.rgb, float3(0.2125, 0.7154, 0.0721));      /// Get luminance
    inColor.rgb = lerp(inColor.a, inColor.rgb, saturation);              /// Saturation
    inColor.rgb = lerp(inColor.rgb, inColor.a * tint.rgb, tint.a);       /// Tint
    inColor.rgb = lerp(middlegray, brightness * inColor.rgb, contrast);  /// Contrast & intensity
    inColor.rgb = lerp(inColor.rgb, fade.rgb, fade.a);                   /// Fade current scene to specified color

    inColor.rgb = saturate(inColor.rgb);

  return inColor;
}