//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//             Contains Tonemapping methods            //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// John Hable: Filmic Uncharted2D                      //
// easyrgb.com: Example of the RGB>XYZ>Yxy color space //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!

// For the Local tone mapping to render as they should, you need to have ENBSeries bloom active.
// You can have BloomAmount set to 0.0 to visually have it off though.


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

string EMPTY_ROW_TONEOP1 = " ";

string TONEOP  = "-------------------TONEMAPPING OPERATORS";
string TONEOP2 = "Change operator through the TECHNIQUE menu at the top";
string TONEOP3 = "Local Operators need Bloom active, Bloom Amount can be 0";

string EMPTY_ROW_TONEOP2 = " ";

//{
string LOGARITHMIC = "Logarithmic Variables";
    float LOGMaxWhite <
        string UIName="LOGARITHMIC: Max White";             string UIWidget="Spinner";    float UIMin=0.01;     float UIMax=25.0;
    > = {1.0};
    float LOGLumSaturation <
        string UIName="LOGARITHMIC: Luminance Saturation";  string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=4.0;
    > = {1.0};

string DRAGOLOG = "Drago Logarithmic Variables";
    float DRAGOMaxWhite <
        string UIName="DRAGO LOG: Max White";               string UIWidget="Spinner";    float UIMin=0.01;     float UIMax=25.0;
    > = {1.0};
    float DRAGOBias <
        string UIName="DRAGO LOG: Bias";                    string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=4.0;
    > = {0.5};
    float DRAGOLumSaturation <
        string UIName="DRAGO LOG: Luminance Saturation";    string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=4.0;
    > = {1.0};

string EXPONENTIAL = "Exponential Variables";
    float EXPMaxWhite <
        string UIName="EXPONENTIAL: Max White";             string UIWidget="Spinner";    float UIMin=0.01;     float UIMax=25.0;
    > = {1.0};
    float EXPLumSaturation <
        string UIName="EXPONENTIAL: Luminance Saturation";  string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=4.0;
    > = {1.0};

string REINHARDB = "Reinhard Basic Variables";
    float REINBMaxWhite <
        string UIName="REINHARDB: Max White";               string UIWidget="Spinner";    float UIMin=0.01;     float UIMax=25.0;
    > = {1.0};
    float REINBLumSaturation <
        string UIName="REINHARDB: Luminance Saturation";    string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=4.0;
    > = {1.0};

string REINHARDM = "Reinhard Modified Variables";
    float REINMMaxWhite <
        string UIName="REINHARDM: Max White";               string UIWidget="Spinner";    float UIMin=0.01;     float UIMax=25.0;
    > = {1.0};
    float REINMLumSaturation <
        string UIName="REINHARDM: Luminance Saturation";    string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=4.0;
    > = {1.0};

string FILMIC_ALU = "Filmic ALU Variables";
    float ALUUpperTone <
        string UIName="FILMIC-ALU: Upper Tone";             string UIWidget="Spinner";    float UIMin=0.0;     float UIMax=10.0;
    > = {0.75};
    float ALUGreyPoint <
        string UIName="FILMIC-ALU: Grey Point";             string UIWidget="Spinner";    float UIMin=0.0;     float UIMax=1.0;
    > = {0.55};
    float ALUMiddleTone <
        string UIName="FILMIC-ALU: Middle Tone";            string UIWidget="Spinner";    float UIMin=0.0;     float UIMax=10.0;
    > = {0.35};
    float ALULowerTone <
        string UIName="FILMIC-ALU: Lower Tone";             string UIWidget="Spinner";    float UIMin=0.0;     float UIMax=10.0;
    > = {0.25};

string FILMIC_UNCHARTED2D = "Filmic Uncharted2D Variables";
    float U2DShoulderStrength <
        string UIName="FILMIC-U2D: Shoulder Strength";      string UIWidget="Spinner";    float UIMin=0.001;    float UIMax=2.0;
    > = {0.16};
    float U2DLinearStrength <
        string UIName="FILMIC-U2D: Linear Strength";        string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=5.0;
    > = {0.1};
    float U2DLinearAngle <
        string UIName="FILMIC-U2D: Linear Angle";           string UIWidget="Spinner";    float UIMin=0.001;    float UIMax=1.0;
    > = {0.1};
    float U2DToeStrength <
        string UIName="FILMIC-U2D: Toe Strength";           string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=2.0;
    > = {0.4};
    float U2DToeNumerator <
        string UIName="FILMIC-U2D: Toe Numerator";          string UIWidget="Spinner";    float UIMin=0.001;    float UIMax=0.5;    float UIStep=0.001;
    > = {0.001};
    float U2DToeDenominator <
        string UIName="FILMIC-U2D: Toe Denominator";        string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=2.0;
    > = {0.3};
    float U2DLinearWhite <
        string UIName="FILMIC-U2D: Linear White";           string UIWidget="Spinner";    float UIMin=0.0;      float UIMax=20.0;
    > = {1.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

// PIXEL SHADERS
float4 PS_TM_Logarithmic(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR                // Global operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output

// Tonemapping, Logarithmic
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = log10(1.0f + pixelLuminance) / log10(1.0f + LOGMaxWhite);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, LOGLumSaturation);

  color.w = 1.0f;
  return color;
}

float4 PS_TM_Logarithmic_Local(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR          // Local operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output
  float4 ebloom = tex2D(_s3, _v0.xy);  /// ENBSeries bloom

// Tonemapping, Logarithmic
    float pixelLuminance      = AvgLuma(color.rgb).w / (1.0f + ebloom.xyz);
    float toneMappedLuminance = log10(1.0f + pixelLuminance) / log10(1.0f + LOGMaxWhite);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, LOGLumSaturation);

  color.w = 1.0f;
  return color;
}


float4 PS_TM_DragoLogarithmic(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR           // Global operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output

// Tonemapping, Drago Logarithmic
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = log10(1.0f + pixelLuminance);
    toneMappedLuminance      /= log10(1.0f + DRAGOMaxWhite);
    toneMappedLuminance      /= log10(2.0f + 8.0f * ((pixelLuminance / DRAGOMaxWhite) * log10(DRAGOBias) / log10(0.5f)));
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, DRAGOLumSaturation);

  color.w = 1.0f;
  return color;
}

float4 PS_TM_DragoLogarithmic_Local(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR     // Local operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output
  float4 ebloom = tex2D(_s3, _v0.xy);  /// ENBSeries bloom

// Tonemapping, Drago Logarithmic
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = log10(1.0f + pixelLuminance);
    toneMappedLuminance      /= log10(1.0f + DRAGOMaxWhite);
    toneMappedLuminance      /= log10(2 + 8 * ((ebloom.xyz / DRAGOMaxWhite) * log10(DRAGOBias) / log10(0.5f)));
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, DRAGOLumSaturation);

  color.w = 1.0f;
  return color;
}


float4 PS_TM_Exponential(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR                // Global operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output

// Tonemapping, Exponential
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = 1.0f - exp(-pixelLuminance / EXPMaxWhite);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, EXPLumSaturation);

  color.w = 1.0f;
  return color;
}

float4 PS_TM_Exponential_Local(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR          // Local operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output
  float4 ebloom = tex2D(_s3, _v0.xy);  /// ENBSeries bloom

// Tonemapping, Exponential
    float pixelLuminance      = AvgLuma(color.rgb).w / (1.0f + ebloom.xyz);
    float toneMappedLuminance = 1.0f - exp(-pixelLuminance / EXPMaxWhite);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, EXPLumSaturation);

  color.w = 1.0f;
  return color;
}


float4 PS_TM_ReinhardBasic(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR              // Global operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output

// Tonemapping, Reinhard Basic
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = pixelLuminance / (pixelLuminance + 1.0f);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, REINBLumSaturation);

  color.w = 1.0f;
  return color;
}

float4 PS_TM_ReinhardBasic_Local(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR        // Local operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output
  float4 ebloom = tex2D(_s3, _v0.xy);  /// ENBSeries bloom

// Tonemapping, Reinhard Basic
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = pixelLuminance / (1.0f + ebloom.xyz);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, REINBLumSaturation);

  color.w = 1.0f;
  return color;
}


float4 PS_TM_ReinhardModified(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR           // Global operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output

// Tonemapping, Reinhard Modified
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = pixelLuminance * (1.0f + pixelLuminance / (REINMMaxWhite * REINMMaxWhite)) / (1.0f + pixelLuminance);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, REINMLumSaturation);

  color.w = 1.0f;
  return color;
}

float4 PS_TM_ReinhardModified_Local(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR     // Local operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output
  float4 ebloom = tex2D(_s3, _v0.xy);  /// ENBSeries bloom

// Tonemapping, Reinhard Modified
    float pixelLuminance      = AvgLuma(color.rgb).w;
    float toneMappedLuminance = pixelLuminance * (1.0f + pixelLuminance / (REINMMaxWhite * REINMMaxWhite)) / (1.0f + ebloom.xyz);
    color.rgb                 = toneMappedLuminance * pow(color.rgb / pixelLuminance, REINMLumSaturation);

  color.w = 1.0f;
  return color;
}


float4 PS_TM_FilmicALU(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR                  // Global operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output

// Tonemapping, Filmic ALU
    color.rgb = max(0, color.rgb - 0.004f);
    color.rgb = (color.rgb * (ALUUpperTone * color.rgb + ALUGreyPoint)) / (color.rgb * (ALUUpperTone * color.rgb + ALUMiddleTone) + ALULowerTone);
    color.rgb = pow(color.rgb, 2.2f);

  color.w = 1.0f;
  return color;
}

float4 PS_TM_FilmicALU_Local(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR            // Local operator technique
{
  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output
  float4 ebloom = tex2D(_s3, _v0.xy);  /// ENBSeries bloom

// Tonemapping, Filmic ALU
    color.rgb = max(0, color.rgb - 0.004f);
    color.rgb = (color.rgb * (ALUUpperTone * color.rgb + ALUGreyPoint / (1.0f + ebloom.xyz))) / (color.rgb * (ALUUpperTone * ebloom.xyz + ALUMiddleTone) + ALULowerTone);
    color.rgb = pow(color.rgb, 2.2f);

  color.w = 1.0f;
  return color;
}


float4 PS_TM_FilmicUncharted2D(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR          // Global operator technique
{
/// Time and Location interpolators ++++///
  float U2D_A = U2DShoulderStrength;    ///
  float U2D_B = U2DLinearStrength;      ///
  float U2D_C = U2DLinearAngle;         ///
  float U2D_D = U2DToeStrength;         ///
  float U2D_E = U2DToeNumerator;        ///
  float U2D_F = U2DToeDenominator;      ///
  float U2D_W = U2DLinearWhite;         ///
///+++++++++++++++++++++++++++++++++++++///

  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output

// Tonemapping, Filmic Uncharted2D
    float3 numerator   = ((color.rgb * (U2D_A * color.rgb + U2D_C * U2D_B) + U2D_D * U2D_E) / (color.rgb * (U2D_A * color.rgb + U2D_B) + U2D_D * U2D_F)) - U2D_E / U2D_F;
    float3 denominator = ((U2D_W * (U2D_A * U2D_W + U2D_C * U2D_B) + U2D_D * U2D_E) / (U2D_W * (U2D_A * U2D_W + U2D_B) + U2D_D * U2D_F)) - U2D_E / U2D_F;
    color.rgb          = numerator / denominator;

  color.w = 1.0f;
  return color;
}

float4 PS_TM_FilmicUncharted2D_Local(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR    // Local operator technique
{
/// Time and Location interpolators ++++///
  float U2D_A = U2DShoulderStrength;    ///
  float U2D_B = U2DLinearStrength;      ///
  float U2D_C = U2DLinearAngle;         ///
  float U2D_D = U2DToeStrength;         ///
  float U2D_E = U2DToeNumerator;        ///
  float U2D_F = U2DToeDenominator;      ///
  float U2D_W = U2DLinearWhite;         ///
///+++++++++++++++++++++++++++++++++++++///

  float4 color  = 0.0f;
  float4 _v0    = 0.0f;
  _v0.xy        = IN.txcoord0.xy;      /// Texture coordinates
  color         = tex2D(_s0, _v0.xy);  /// ENBSeries color output
  float4 ebloom = tex2D(_s3, _v0.xy);  /// ENBSeries bloom

// Tonemapping, Filmic Uncharted2D
    float3 numerator   = ((color.rgb * (U2D_A * color.rgb + U2D_C * U2D_B) + U2D_D * U2D_E) / (color.rgb * (U2D_A * color.rgb + U2D_B) + U2D_D * U2D_F)) - U2D_E / U2D_F;
    float3 denominator = (((U2D_W / (1.0f + ebloom.xyz)) * (U2D_A * (U2D_W / (1.0f + ebloom.xyz)) + U2D_C * U2D_B) + U2D_D * U2D_E) / ((U2D_W / (1.0f + ebloom.xyz)) * (U2D_A * (U2D_W / (1.0f + ebloom.xyz)) + U2D_B) + U2D_D * U2D_F)) - U2D_E / U2D_F;
    color.rgb          = numerator / denominator;

  color.w = 1.0f;
  return color;
}




// TECHNIQUES
technique Shader_TM_1 <string UIName="TM: Logarithmic";>                // Global operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_Logarithmic();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
technique Shader_TM_8 <string UIName="TM: Logarithmic Local";>          // Local operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_Logarithmic_Local();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}


technique Shader_TM_2 <string UIName="TM: Drago Logarithmic";>          // Global operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_DragoLogarithmic();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
technique Shader_TM_9 <string UIName="TM: Drago Logarithmic Local";>    // Local operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_DragoLogarithmic_Local();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}


technique Shader_TM_3 <string UIName="TM: Exponential";>                // Global operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_Exponential();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
technique Shader_TM_10 <string UIName="TM: Exponential Local";>         // Local operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_Exponential_Local();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}


technique Shader_TM_4 <string UIName="TM: Reinhard Basic";>             // Global operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_ReinhardBasic();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
technique Shader_TM_11 <string UIName="TM: Reinhard Basic Local";>      // Local operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_ReinhardBasic_Local();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}


technique Shader_TM_5 <string UIName="TM: Reinhard Modified";>          // Global operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_ReinhardModified();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
technique Shader_TM_12 <string UIName="TM: Reinhard Modified Local";>   // Local operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_ReinhardModified_Local();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}


technique Shader_TM_6 <string UIName="TM: Filmic-ALU";>                 // Global operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_FilmicALU();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
technique Shader_TM_13 <string UIName="TM: Filmic-ALU Local";>          // Local operator technique
{
   pass p0
   {
      VertexShader = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_FilmicALU_Local();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}


technique Shader_TM_7 <string UIName="TM: Filmic-Uncharted2D";>         // Global operator technique
{
   pass p0
   {
      VertexShader  = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_FilmicUncharted2D();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
technique Shader_TM_14 <string UIName="TM: Filmic-Uncharted2D Local";>  // Local operator technique
{
   pass p0
   {
      VertexShader  = compile vs_3_0 VS_Quad();
      PixelShader  = compile ps_3_0 PS_TM_FilmicUncharted2D_Local();

      ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
      ZEnable=FALSE;
      ZWriteEnable=FALSE;
      CullMode=NONE;
      AlphaTestEnable=FALSE;
      AlphaBlendEnable=FALSE;
      SRGBWRITEENABLE=FALSE;
   }
}
