//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains HDR Toning                 //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// Jim Hejl: Filmic ALU                                //
// Richard Burgess-Dawson: Filmic ALU                  //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_HDRT <
    string UIName = "Enable HDR Toning";
  > = {true};
  bool ADAPT_HDRT <
    string UIName = "Enable Adaptation";
  > = {true};
//{
    float fMaxLumLoc <
      string UIName="HDR Reinhard: Max Luminance";          string UIWidget="Spinner";  float UIMin=0.01;
    > = {1.0};
    float fLumBright <
      string UIName="HDR Reinhard: Luminance Intensity";    string UIWidget="Spinner";  float UIMin=0.01;
    > = {1.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msHDRTonemap(float3 inColor, float3 inBlur, float3 inAdaptation)
{
  if (ENABLE_HDRT==true)
  {
  /// Color Space Conversion STARTS
    float3 XYZ = RGBtoXYZ(inColor.rgb);  /// Color space conversion, from RGB to XYZ
    float3 Yxy = XYZtoYxy(XYZ.xyz);      /// Color space conversion, from XYZ to Yxy

  /// Simple Brightness, applied to Y/Luminance channel of Yxy color space
    Yxy.r *= fLumBright;

  /// Reinhard Tonemapping, applied to Y/Luminance channel of Yxy color space
    if (ADAPT_HDRT==true)  Yxy.r = (Yxy.r * (Yxy.r + Yxy.r / 1.0f)) / (inAdaptation + inBlur);  /// Reinhard Adaptation, will be converted to Local Adaptation when ENB Bloom is available
    Yxy.r = (Yxy.r * (1.0f + Yxy.r / fMaxLumLoc)) / (Yxy.r + 1);

  /// Color Space Conversion ENDS
    XYZ.xyz      = YxytoXYZ(XYZ.xyz, Yxy.xyz);  /// Color space conversion, from Yxy to XYZ
    inColor.rgb  = XYZtoRGB(XYZ.xyz);           /// Color space conversion, from XYZ to RGB
  }

  return inColor;
}