//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains HDR Curve                  //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// Jim Hejl: Filmic ALU                                //
// Richard Burgess-Dawson: Filmic ALU                  //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_HDRALU <
    string UIName = "Enable HDR Filmic ALU";
  > = {true};
  bool ADAPT_HDRALU <
    string UIName = "Enable Adaptation";
  > = {true};
//{
    float fKeyValue <
      string UIName="HDR-ALU: Exposure";     string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;
    > = {1.0};
    float fUpperTone <
      string UIName="HDR-ALU: Upper Tone";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
    float fGreyPoint <
      string UIName="HDR-ALU: Grey Point";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1.0;
    > = {1.0};
    float fMiddleTone <
      string UIName="HDR-ALU: Middle Tone";  string UIWidget="Spinner";  float UIMin=0.0;  //float UIMax=25.0;
    > = {1.0};
    float fLowerTone <
      string UIName="HDR-ALU: Lower Tone";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msHDRTonemap(float3 inColor, float3 inBlur, float3 inAdaptation)
{
  if (ENABLE_HDRALU==true)
  {
  /// Color Space Conversion STARTS
    float3 XYZ = RGBtoXYZ(inColor.rgb);  /// Color space conversion, from RGB to XYZ
    float3 Yxy = XYZtoYxy(XYZ.xyz);      /// Color space conversion, from XYZ to Yxy

    Yxy.r *= fKeyValue;  /// Simple Brightness adjustment

  /// Tonemapping, Global Filmic ALU
    if (ADAPT_HDRALU==true)  Yxy.r = (Yxy.r * (Yxy.r + Yxy.r / 1.0f)) / (inAdaptation + inBlur);  /// Reinhard Adaptation, will be converted to Local Adaptation when ENB Bloom is available
    Yxy.r = (Yxy.r * (fUpperTone * Yxy.r + fGreyPoint)) / (Yxy.r * (fUpperTone + fMiddleTone) + fLowerTone);  /// Tonemapping with Highlight and Shadow controls as well as "max allowed brightness"

  /// Color Space Conversion ENDS
    XYZ.xyz      = YxytoXYZ(XYZ.xyz, Yxy.xyz);  /// Color space conversion, from Yxy to XYZ
    inColor.rgb  = XYZtoRGB(XYZ.xyz);           /// Color space conversion, from XYZ to RGB
  }

  return inColor;
}