//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains HDR Combo                  //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// John Hable: Filmic Uncharted2D                      //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_HDRU2D <
    string UIName = "Enable HDR Filmic Uncharted2D";
  > = {true};
  bool ADAPT_HDRU2D <
    string UIName = "Enable Adaptation";
  > = {true};
//{
    float fKeyValue <
        string UIName="HDR-U2D: Exposure";           string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;  float UIStep=0.001;
    > = {1.0};
    float ShoulderStrength <
        string UIName="HDR-U2D: Shoulder Strength";  string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=2.0;
    > = {0.16};
    float LinearStrength <
        string UIName="HDR-U2D: Linear Strength";    string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;
    > = {0.2};
    float LinearAngle <
        string UIName="HDR-U2D: Linear Angle";       string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=1.0;
    > = {0.1};
    float ToeStrength <
        string UIName="HDR-U2D: Toe Strength";       string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=2.0;
    > = {0.2};
    float ToeNumerator <
        string UIName="HDR-U2D: Toe Numerator";      string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=0.5;  float UIStep=0.001;
    > = {0.001};
    float ToeDenominator <
        string UIName="HDR-U2D: Toe Denominator";    string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=2.0;
    > = {0.2};
    float LinearWhite <
        string UIName="HDR-U2D: Linear White";       string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {1.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msHDRTonemap(float3 inColor, float3 inBlur, float3 inAdaptation)
{
///++++++++++++++++++++++++++++++///
  float A = ShoulderStrength;    ///
  float B = LinearStrength;      ///
  float C = LinearAngle;         ///
  float D = ToeStrength;         ///
  float E = ToeNumerator;        ///
  float F = ToeDenominator;      ///
  float W = LinearWhite;         ///
///++++++++++++++++++++++++++++++///

  if (ENABLE_HDRU2D==true)
  {
  /// Color Space Conversion STARTS
    float3 XYZ = RGBtoXYZ(inColor.rgb);  /// Color space conversion, from RGB to XYZ
    float3 Yxy = XYZtoYxy(XYZ.xyz);      /// Color space conversion, from XYZ to Yxy

  /// Tonemapping and Exposure
    Yxy.r *= fKeyValue;
    if (ADAPT_HDRU2D==true)
    {
      float lumScale = (Yxy.r * (Yxy.r + Yxy.r / 1.0f)) / (inAdaptation + inBlur);  /// Reinhard Adaptation, will be converted to Local Adaptation when ENB Bloom is available
      Yxy.r  = ((lumScale * (A * lumScale + C * B) + D * E) / (lumScale * (A * lumScale + B) + D * F)) - E / F;
      Yxy.r /= ((W * (A * W + C * B) + D * E) / (W * (A * W + B) + D * F)) - E / F;
    }
    else if (ADAPT_HDRU2D==false)
    {
      Yxy.r  = ((Yxy.r * (A * Yxy.r + C * B) + D * E) / (Yxy.r * (A * Yxy.r + B) + D * F)) - E / F;
      Yxy.r /= ((W * (A * W + C * B) + D * E) / (W * (A * W + B) + D * F)) - E / F;
    }

  /// Color Space Conversion ENDS
    XYZ.xyz      = YxytoXYZ(XYZ.xyz, Yxy.xyz);  /// Color space conversion, from Yxy to XYZ
    inColor.rgb  = XYZtoRGB(XYZ.xyz);           /// Color space conversion, from XYZ to RGB
  }

  return inColor;
}