//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//               Contains Local Contrast               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// Jim Hejl: Filmic ALU                                //
// Richard Burgess-Dawson: Filmic ALU                  //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

//{
    float fLocalHighlights <
      string UIName="CONTRAST: Highlights";  string UIWidget="Spinner";  float UIMin=-1.0;  float UIMax=3.0;
    > = {0.0};
    float fLocalShadows <
      string UIName="CONTRAST: Shadows";     string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=3.0;
    > = {0.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msLocalContrast(float3 inColor, float3 inBlur, float3 inAdaptation)
{
// Color Space Conversion STARTS
  float3 XYZ = RGBtoXYZ(inColor.rgb);  /// Color space conversion, from RGB to XYZ
  float3 Yxy = XYZtoYxy(XYZ.xyz);      /// Color space conversion, from XYZ to Yxy

/// Adaptive Local Contrast
    float lcBlurLuma  = inBlur;
    float lcColorLuma = Yxy.r;
    float lcNewLuma   = lcColorLuma;
    if (lcColorLuma > lcBlurLuma)
    {
        lcNewLuma = lcColorLuma + (lcColorLuma - lcBlurLuma) * (1 - (lcColorLuma - lcBlurLuma) / lcColorLuma) * (fLocalHighlights / (inAdaptation + 1));
    }
    else if(lcColorLuma < lcBlurLuma)
    {
        lcNewLuma = lcColorLuma - (lcBlurLuma - lcColorLuma) * (1 - (lcBlurLuma - lcColorLuma) / lcBlurLuma) * (fLocalShadows / (inAdaptation + 1));
    };
    Yxy.r *= lcNewLuma / lcColorLuma;

// Color Space Conversion ENDS
  XYZ.xyz      = YxytoXYZ(XYZ.xyz, Yxy.xyz);  /// Color space conversion, from Yxy to XYZ
  inColor.rgb  = XYZtoRGB(XYZ.xyz);           /// Color space conversion, from XYZ to RGB

  return inColor;
}