//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//        Contains Procedural Color Corrections        //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Ceejay.dk: Author of initial Vibrance, Saturation   //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx or effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
#ifdef E_CC_PROCEDURAL
    float fHighlights <
        string UIName="PCC: Highlights";            string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1.0;  float UIStep=0.001;
    > = {0.0};
    float fShadows <
        string UIName="PCC: Shadows";               string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1.0;  float UIStep=0.001;
    > = {0.0};
    float fSaturation <
        string UIName="PCC: Saturation";            string UIWidget="Spinner";  float UIMin=-1.0;  float UIMax=2.0;
    > = {0.0};
    float fVibrance <
        string UIName="PCC: Vibrance";              string UIWidget="Spinner";  float UIMin=-1.0;  float UIMax=1.0;  float UIStep=0.001;
    > = {0.0};
    float3 fVibrance_RGB <
        string UIName="PCC: Vibrance RGB Balance";  string UIWidget="Spinner";  float UIMin=-10.0;  float UIMax=10.0;
    > = {0.0, 0.0, 0.0};
#endif


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msPCC(float3 inColor)
{
#ifdef E_CC_PROCEDURAL
/// Temp variables
    float3 tempgray = AvgLuma(inColor).w;

/// Highlights and Shadows
    inColor.rgb += min(tempgray.rgb * fHighlights, 1.0f) * max(tempgray.rgb * fHighlights, 0.0f);
    inColor.rgb -= min(tempgray.rgb * fShadows, 0.05f) * max(tempgray.rgb * fShadows, 1.0f);

/// Vibrance
    float3 Vibrance_coeff  = float3((fVibrance_RGB.xyz * 2) * fVibrance);
    float max_color        = max(inColor.r, max(inColor.g, inColor.b));  /// Find the strongest color
    float min_color        = min(inColor.r, min(inColor.g, inColor.b));  /// Find the weakest color
    float color_saturation = max_color - min_color;                      /// The difference between the two is the saturation
    inColor.rgb            = lerp(tempgray, inColor.rgb, (1.0 + (Vibrance_coeff.xyz * (1.0 - (sign(Vibrance_coeff.xyz) * color_saturation))))); /// Extrapolate between luma and original by 1 + (1-saturation) - current

/// Saturation
    float3 diffcolor = inColor.rgb - tempgray;
    inColor.rgb      = (inColor.rgb + diffcolor.rgb * fSaturation) / (1.0f + (diffcolor.rgb * fSaturation));
#endif

  return inColor;
}