//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains ENB Palette                        //
//           used by the Modular Shader files                  //
//-----------------------CREDITS-------------------------------//
// JawZ: Author and developer of this file                     //
// kingeric1992: 3D LUT code and texture atlas implementation  //
// tktk: original DNI separation                               //
// TomDean: adaption of the DNI separation for FO4             //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the elepHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
#ifdef E_CC_PROCEDURAL
  bool ENABLE_3DLUT <
    string UIName = "Enable 3D LUT";
  > = {false};
  
#endif


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //

Texture2D TextureLutD < 
  string UIName = "3DLutDay";  string ResourceName = "enbseries/Modular Shaders/LUT.png"; >;

SamplerState SamplerLut {
   Filter=MIN_MAG_MIP_LINEAR;  AddressU=Clamp;  AddressV=Clamp;
};


// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 ms3DLUT(float3 inColor)
{
#ifdef E_CC_PROCEDURAL
if(ENABLE_3DLUT==true)
{
    float2 CLut_pSize = {0.00390625, 0.0625};
    inColor.rgb         = saturate(inColor.rgb) * 15;
   
    float3 CLut_UV;   
    CLut_UV.z = floor(inColor.z);
    inColor.z  -= CLut_UV.z;
    inColor.xy  = (inColor.xy + 0.5) * CLut_pSize;
    inColor.x  += CLut_UV.z * CLut_pSize.y;
    inColor.y  *= 0.3333333333333333;

    CLut_UV.x = inColor.x;
    CLut_UV.z = CLut_UV.x + CLut_pSize.y;
    CLut_UV.y = inColor.y + 0 * 0.3333333333333333;
    float3 lutcolor_D = lerp(TextureLutD.SampleLevel(Sampler1, CLut_UV.xy, 0).rgb, TextureLutD.SampleLevel(Sampler1, CLut_UV.zy, 0).rgb, inColor.z);
    CLut_UV.y = inColor.y + 1 * 0.3333333333333333;
    float3 lutcolor_N = lerp(TextureLutD.SampleLevel(Sampler1, CLut_UV.xy, 0).rgb, TextureLutD.SampleLevel(Sampler1, CLut_UV.zy, 0).rgb, inColor.z);
    CLut_UV.y = inColor.y + 2 * 0.3333333333333333;
    float3 lutcolor_I = lerp(TextureLutD.SampleLevel(Sampler1, CLut_UV.xy, 0).rgb, TextureLutD.SampleLevel(Sampler1, CLut_UV.zy, 0).rgb, inColor.z);

    inColor.rgb         = lerp(lerp( lutcolor_N, lutcolor_D, ENightDayFactor), lutcolor_I, EInteriorFactor);
}
#endif
  return saturate(inColor.rgb);
}