//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//            Contains ENB Post-Process v2             //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Boris: Initial author of ENB post-process method    //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_PP2 <
    string UIName = "Enable Post Process v2";
  > = {false};
//{
    float EAdaptationMinV2 <
        string UIName="PP2: Adaptation Min";              string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=100.0;  float UIStep=0.001;
    > = {0.05};
    float EAdaptationMaxV2 <
        string UIName="PP2: Adaptation Max";              string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=100.0;  float UIStep=0.001;
    > = {0.05};
    float EBrightnessV2 <
        string UIName="PP2: Brightness";                  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
    float EIntensityContrastV2 <
        string UIName="PP2: Intensity Contrast";          string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
    float EColorSaturationV2 <
        string UIName="PP2: Color Saturation";            string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
    float EToneMappingOversaturationV2 <
        string UIName="PP2: Tonemapping Oversaturation";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1000.0;
    > = {180.0};
    float EToneMappingCurveV2 <
        string UIName="PP2: Tonemapping Curve";           string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {8.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 enbPP(float3 inColor, float3 inAdaptation)
{
  if (ENABLE_PP2==true)
  {
    float grayadaptation = AvgLuma(inAdaptation.xyz).y;
      grayadaptation     = max(grayadaptation, 0.0);
      grayadaptation     = min(grayadaptation, 50.0);
      inColor.xyz        = inColor.xyz / (grayadaptation * EAdaptationMaxV2 + EAdaptationMinV2);

    inColor.xyz  *= EBrightnessV2;
    inColor.xyz  += 0.000001;
    float3 xncol  = normalize(inColor.xyz);
    float3 scl    = inColor.xyz / xncol.xyz;
    scl           = pow(scl, EIntensityContrastV2);
    xncol.xyz     = pow(xncol.xyz, EColorSaturationV2);
    inColor.xyz   = scl * xncol.xyz;
    float lumamax = EToneMappingOversaturationV2;
    inColor.xyz   = (inColor.xyz * (1.0 + inColor.xyz / lumamax)) / (inColor.xyz + EToneMappingCurveV2);
  }

  return inColor;
}