//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//            Contains ENB Post-Process v4             //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Boris: Initial author of ENB post-process method    //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - DNSrSsEIFactor(WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight)
 * - DNEIFactor(NightExt, DayExt, NightInt, DayInt)                       *
 * - DNIFactor(NightExt, DayExt, Int)                                     *
 * - DNSrSsEIFactor(_c3, WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight) // ELE only!
 * - DNIDFactor(_c3, NightExt, DayExt, Interior, Dungeon) // ELE only!    *
 * - IDFactor(_c3, Interior, Dungeon)                     // ELE only!    *
 * - DunFactor(_c3, NightDayInt, Dungeon)                 // ELE only!    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_PP4 <
    string UIName = "Enable Post Process v4";
  > = {false};
//{
    float EAdaptationMinV4 <
        string UIName="PP4: Adaptation Min";                string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=100.0;  float UIStep=0.001;
    > = {0.2};
    float EAdaptationMaxV4 <
        string UIName="PP4: Adaptation Max";                string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=100.0;  float UIStep=0.001;
    > = {0.125};
    float EBrightnessCurveV4 <
        string UIName="PP4: Brightness Curve";              string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1000.0;
    > = {0.7};
    float EBrightnessMultiplierV4 <
        string UIName="PP4: Brightness Multiplier";         string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {0.45};
    float EBrightnessToneMappingCurveV4 <
        string UIName="PP4: Brightness Tonemapping Curve";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {0.5};
//}

// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 enbPP(float3 inColor, float3 inAdaptation)
{
  if (ENABLE_PP4==true)
  {
    float grayadaptation = AvgLuma(inAdaptation.xyz).y;
      grayadaptation     = max(grayadaptation, 0.0);
      grayadaptation     = min(grayadaptation, 50.0);
      inColor.xyz        = inColor.xyz / (grayadaptation * EAdaptationMaxV4 + EAdaptationMinV4);

    float Y = dot(inColor.xyz, float3(0.299, 0.587, 0.114));       /// 0.299 * R + 0.587 * G + 0.114 * B;
    float U = dot(inColor.xyz, float3(-0.14713, -0.28886, 0.436)); /// -0.14713 * R - 0.28886 * G + 0.436 * B;
    float V = dot(inColor.xyz, float3(0.615, -0.51499, -0.10001)); /// 0.615 * R - 0.51499 * G - 0.10001 * B;
      Y = pow(Y, EBrightnessCurveV4);
      Y = Y * EBrightnessMultiplierV4;
      Y = Y / (Y + EBrightnessToneMappingCurveV4);     // Deactivate, if using the lowest two rows
    float desaturatefact = saturate(Y * Y * Y * 1.7);  // Deactivate, if using the lowest two rows
      U = lerp(U, 0.0, desaturatefact);                // Deactivate, if using the lowest two rows
      V = lerp(V, 0.0, desaturatefact);                // Deactivate, if using the lowest two rows
      inColor.xyz = V * float3(1.13983, -0.58060, 0.0) + U * float3(0.0, -0.39465, 2.03211) + Y;

/// Deactivate these two code lines below and activate the marked code lines above for a different "Max White" control
//      inColor.xyz = max(inColor.xyz, 0.0);
//      inColor.xyz = inColor.xyz / (inColor.xyz + EBrightnessToneMappingCurveV4);
  }

  return inColor;
}