//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains ENB Palette                //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Boris: Initial author of ENB Palette method         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
#ifdef E_CC_PROCEDURAL
  bool ENABLE_PALETTE <
    string UIName = "Enable Palette";
  > = {false};
#endif


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //

Texture2D TexturePalette <    /// Palette texture map
  string UIName = "Palette";  string ResourceName = "enbseries/Modular Shaders/enbpalette.bmp";
>;

SamplerState SamplerPalette {
   Filter=MIN_MAG_MIP_LINEAR;  AddressU=Clamp;  AddressV=Clamp;
};



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 enbPalette(float3 inColor, float3 inAdaptation)
{
#ifdef E_CC_PROCEDURAL
  if (ENABLE_PALETTE==true)
  {
  float3 palette;
  float4 coord = 0.0;

    inAdaptation = (inAdaptation / (inAdaptation + 1.0));
//    inAdaptation = max(inAdaptation.x, max(inAdaptation.y, inAdaptation.z));

    inColor.xyz = saturate(inColor.xyz);
    coord.y     = inAdaptation.r;
    coord.x     = inColor.r;
    palette.r   = TexturePalette.SampleLevel(SamplerPalette, coord, 0).r;
    coord.x     = inColor.g;
    coord.y     = inAdaptation.g;
    palette.g   = TexturePalette.SampleLevel(SamplerPalette, coord, 0).g;
    coord.x     = inColor.b;
    coord.y     = inAdaptation.b;
    palette.b   = TexturePalette.SampleLevel(SamplerPalette, coord, 0).b;

    inColor.rgb = palette.rgb;
  }
#endif

  return inColor;
}