//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//        Contains ExtendedFX Sharpening Depth        //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Initial depth based Sharpening code         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the effect.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row1 <
  string UIName="A--------------------------A";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

//{
    bool ENABLE_SHARPENING <
        string UIName = "Enable Sharpening";
    > = {false};
    bool SHOW_EDGES <
        string UIName = "Show only the Sharpened Edges";
    > = {false};
    bool SMOOTH_EDGES <
        string UIName = "Use Edge Smoothing";
    > = {true};
    bool LUMA_SHARPEN <
        string UIName = "Use Luma Sharpen";
    > = {true};

    float fBlurSigma <
        string UIName="Sharpening: Radius";           string UIWidget="Spinner";  float UIMin=0.01;  float UIMax=2.0;  float UIStep=0.001;
    > = {0.44};
    int fFarDepth <
        string UIName="Sharpening: Far Depth";        string UIWidget="Spinner";  int UIMin=0;  int UIMax=100000;
    > = {300};
    float fSharpening <
        string UIName="Sharpening: Amount";           string UIWidget="Spinner";  float UIMin=0.01;  float UIMax=5.0;  float UIStep=0.001;
    > = {3.0};
    int fThreshold <
        string UIName="Sharpening: Threshold";        string UIWidget="Spinner";  int UIMin=0;  int UIMax=255;
    > = {0};
    float fLimiter <
        string UIName="Sharpening: HL Limiter";       string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1.0;  float UIStep=0.001;
    > = {1.0};
    float fSmooth <
        string UIName="Edge Smoothing: Intensity";    string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1.0;  float UIStep=0.001;
    > = {1.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float4 msGaussianH(float4 inColor, float2 inCoord)
{
if (ENABLE_SHARPENING==false) return float4(TextureColor.Sample(Sampler0, inCoord.xy));

  inColor     = TextureOriginal.Sample(Sampler1, inCoord.xy);
  float Depth = TextureDepth.Sample(Sampler0, inCoord.xy ).x;

    float px       = ScreenSize.y; 
    float linDepth = linearDepth(Depth, 0.5f, fFarDepth);

    float SigmaSum     = 0.0f;
    float sampleOffset = 1.0f;

/// Gaussian
      float fBlurSigma1 = max( fBlurSigma * ( 1.0f - linDepth ), 0.3f );
    float3 Sigma;
      Sigma.x = 1.0f / ( sqrt( 2.0f * PI ) * fBlurSigma1 );
      Sigma.y = exp( -0.5f / ( fBlurSigma1 * fBlurSigma1 ));
      Sigma.z = Sigma.y * Sigma.y;

/// Center weight
    inColor  *= Sigma.x;
    SigmaSum += Sigma.x;
    Sigma.xy *= Sigma.yz;

    for(int i = 0; i < 7; ++i)
    {
      inColor     += TextureColor.Sample(Sampler1, inCoord.xy + float2(sampleOffset * px, 0.0)) * Sigma.x;
      inColor     += TextureColor.Sample(Sampler1, inCoord.xy - float2(sampleOffset * px, 0.0)) * Sigma.x;
      SigmaSum    += (2.0f * Sigma.x);
      sampleOffset = sampleOffset + 1.0f;
      Sigma.xy    *= Sigma.yz;
    }
 
  inColor.xyz /= SigmaSum;
  inColor.w    = 1.0f;
  return inColor;
}

float4 msGaussianV(float4 inColor, float2 inCoord)
{
if (ENABLE_SHARPENING==false) return float4(TextureColor.Sample(Sampler1, inCoord.xy));

  float sHeight  = ScreenSize.x * ScreenSize.w;
  float py       = 1.0 / sHeight;
  float Depth    = TextureDepth.Sample(Sampler0, inCoord.xy ).x;
  float linDepth = linearDepth( Depth, 0.5f, fFarDepth );

  float SigmaSum     = 0.0f;
  float sampleOffset = 1.0f;

/// Gaussian
      float fBlurSigma1 = max( fBlurSigma * ( 1.0f - linDepth ), 0.3f );
    float3 Sigma;
      Sigma.x = 1.0f / ( sqrt( 2.0f * PI ) * fBlurSigma1 );
      Sigma.y = exp( -0.5f / ( fBlurSigma1 * fBlurSigma1 ));
      Sigma.z = Sigma.y * Sigma.y;

/// Center weight
    inColor   = TextureColor.Sample(Sampler1, inCoord.xy);
    inColor  *= Sigma.x;
    SigmaSum += Sigma.x;
    Sigma.xy *= Sigma.yz;

    for(int i = 0; i < 7; ++i)
    {
      inColor     += TextureColor.Sample(Sampler1, inCoord.xy + float2(0.0, sampleOffset * py)) * Sigma.x;
      inColor     += TextureColor.Sample(Sampler1, inCoord.xy - float2(0.0, sampleOffset * py)) * Sigma.x;
      SigmaSum    += ( 2.0f * Sigma.x );
      sampleOffset = sampleOffset + 1.0f;
      Sigma.xy    *= Sigma.yz;
    }

  inColor.xyz /= SigmaSum;
  inColor.w    = 1.0f;
  return inColor;
}

float4 msEdges(float4 inColor, float2 inCoord)
{
if (ENABLE_SHARPENING==false) return float4(TextureColor.Sample(Sampler1, inCoord.xy));

  float4 orig    = TextureOriginal.Sample(Sampler1, inCoord.xy);
  float4 blurred = TextureColor.Sample(Sampler1, inCoord.xy);

/// Find edges
    orig.xyz           = saturate( orig.xyz );
    blurred.xyz        = saturate( blurred.xyz );
    float3 Edges       = max( saturate( orig.xyz - blurred.xyz ) - fThreshold, 0.0f );
    float3 invBlur     = saturate( 1.0f - blurred.xyz );
    float3 originvBlur = saturate( orig.xyz + invBlur.xyz );
    float3 invOrigBlur = max( saturate( 1.0f - originvBlur.xyz ) - fThreshold, 0.0f );

    float3 edges = max(( saturate( fSharpening * Edges.xyz )) - ( saturate( fSharpening * invOrigBlur.xyz )), 0.0f );

  inColor.xyz = edges.xyz;
  inColor.w   = 1.0f;
  return inColor;
}

float4 msSharpen1(float4 inColor, float2 inCoord)
{
if (ENABLE_SHARPENING==false) return float4(TextureColor.Sample(Sampler1, inCoord.xy));

    float4 edges = TextureColor.Sample(Sampler1, inCoord.xy);

/// Smooth out edges (reduce aliasing) - expensive, likely
    if (SMOOTH_EDGES==false) return edges;

    float px = ScreenSize.y;

    float SigmaSum     = 0.0f;
    float sampleOffset = 1.0f;

/// Gaussian
    float BlurSigma = fSmooth;
    float3 Sigma;
    Sigma.x         = 1.0f / ( sqrt( 2.0f * PI ) * BlurSigma );
    Sigma.y         = exp( -0.5f / ( BlurSigma * BlurSigma ));
    Sigma.z         = Sigma.y * Sigma.y;

/// Center weight
    edges    *= Sigma.x;
    SigmaSum += Sigma.x;
    Sigma.xy *= Sigma.yz;

    for(int i = 0; i < 5; ++i)
    {
      edges       += TextureColor.Sample(Sampler1, inCoord.xy + float2(sampleOffset * px, 0.0)) * Sigma.x;
      edges       += TextureColor.Sample(Sampler1, inCoord.xy - float2(sampleOffset * px, 0.0)) * Sigma.x;
      SigmaSum    += ( 2.0f * Sigma.x );
      sampleOffset = sampleOffset + 1.0f;
      Sigma.xy    *= Sigma.yz;
    }

  inColor.xyz = edges.xyz / SigmaSum;
  inColor.w   = 1.0f;
  return inColor;
}

float4 msSharpen2(float4 inColor, float2 inCoord)
{
if (ENABLE_SHARPENING==false) return float4(TextureColor.Sample(Sampler1, inCoord.xy));

  float4 orig  = TextureOriginal.Sample(Sampler1, inCoord.xy);
  float4 edges = TextureColor.Sample(Sampler1, inCoord.xy);

/// Smooth out edges (reduce aliasing) - expensive, likely
    float sHeight = ScreenSize.x * ScreenSize.w;
    float py      = 1.0 / sHeight;

    if (SMOOTH_EDGES==true)
    {
      float SigmaSum     = 0.0f;
      float sampleOffset = 1.0f;

/// Gaussian
      float BlurSigma = fSmooth;
      float3 Sigma;
      Sigma.x = 1.0f / ( sqrt( 2.0f * PI ) * BlurSigma );
      Sigma.y = exp( -0.5f / ( BlurSigma * BlurSigma ));
      Sigma.z = Sigma.y * Sigma.y;

/// Center weight
      edges    *= Sigma.x;
      SigmaSum += Sigma.x;
      Sigma.xy *= Sigma.yz;

      for(int i = 0; i < 5; ++i)
      {
        edges       += TextureColor.Sample(Sampler1, inCoord.xy + float2(0.0, sampleOffset * py)) * Sigma.x;
        edges       += TextureColor.Sample(Sampler1, inCoord.xy - float2(0.0, sampleOffset * py)) * Sigma.x;
        SigmaSum    += ( 2.0f * Sigma.x );
        sampleOffset = sampleOffset + 1.0f;
        Sigma.xy    *= Sigma.yz;
      }

      edges.xyz /= SigmaSum;
    }

    if (SHOW_EDGES==true)
    {
      inColor.w = 1.0f;
        if(LUMA_SHARPEN==true)
        {
          inColor.xyz = min( dot( edges.xyz, float3( 0.2126, 0.7152, 0.0722 )), fLimiter );
        } else {
          inColor.xyz = edges.xyz * fLimiter;
        }
      return inColor;
    }

    if (LUMA_SHARPEN==true)
    {
      float3 blend = saturate( orig.xyz + min( dot( edges.xyz, float3( 0.2126, 0.7152, 0.0722 )), fLimiter ));
      inColor.xyz  = BlendLuma( orig.xyz, blend.xyz );
    } else {
      inColor.xyz  = saturate( orig.xyz + ( edges.xyz * fLimiter ));
    }

  inColor.w = 1.0f;
  return inColor;
}