//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//        Contains ExtendedFX Sharpening Depth        //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Initial depth based Sharpening code         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the effect.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row1 <
  string UIName="A--------------------------A";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

//{
    bool ENABLE_SHARPENING <
        string UIName = "Enable Sharpening";
    > = {false};
    bool ENABLE_DEPTH <
        string UIName = "Enable Depth Sharpening";
    > = {false};
    bool SHOW_EDGES <
        string UIName = "Show only the Sharpened Edges";
    > = {false};
    int CHOOSE_BLUR <
        string UIName = "Choose Blur Method";  int UIMin=1;  int UIMax=4;
    > = {3};

    float fSharpStrength <
        string UIName="Sharpening: Strength";     string UIWidget="Spinner";  float UIMin=0.1;  float UIMax=30.0;  float UIStep=0.001;
    > = {5.0};
    float fSharpClamp <
        string UIName="Sharpening: Limiter";      string UIWidget="Spinner";  float UIMin=0;  float UIMax=1;  float UIStep=0.001;
    > = {1.0};
    int fFromDepth <
      string UIName="Sharpening: From Depth";     string UIWidget="Spinner";  int UIMin=0.0;  int UIMax=100000.0;
    > = {300.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //

/// BT.709 & sRBG luma coefficient (Monitors and HD Television)
#define CoefLuma float3(0.2126, 0.7152, 0.0722)

/// 
  #define px  ScreenSize.y

/// 
  #define sHeight  ScreenSize.x * ScreenSize.w
  #define py       1.0 / sHeight

/// Shouldn't really be used as an annotation!
  #define fOffsetBias 1.0


// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 LumaSharpenPass(float3 inColor, float2 tex)
{
if (ENABLE_SHARPENING==false) return float4(TextureColor.Sample(Sampler0, tex.xy));

// Get the original pixel
  float3 ori = TextureColor.Sample(Sampler0, tex).rgb;       // ori = original pixel

// Combining the strength and luma multipliers
  float3 sharp_strength_luma = (CoefLuma * fSharpStrength); //I'll be combining even more multipliers with it later on

// Intialize variable
  float3 blur_ori;

// Simple depth implementation example
  float Depth        = TextureDepth.Sample(Sampler0, tex.xy ).x;
  float linDepthFrom = linearDepth(Depth, 0.5f, fFromDepth);


   /*-----------------------------------------------------------.
  /                             Blur                            /
  '-----------------------------------------------------------*/

// Pattern 1 -- A (fast) 7 tap gaussian using only 2+1 texture fetches.
  if (CHOOSE_BLUR==1)
  {
    blur_ori  = TextureColor.Sample(Sampler0, tex + (float2(px,py) / 3.0) * fOffsetBias).rgb;    // North West
    blur_ori += TextureColor.Sample(Sampler0, tex + (float2(-px,-py) / 3.0) * fOffsetBias).rgb;  // South East

    blur_ori /= 2;               //Divide by the number of texture fetches
    sharp_strength_luma *= 1.5;  // Adjust strength to aproximate the strength of pattern 2
  }

  // -- Pattern 2 -- A 9 tap gaussian using 4+1 texture fetches.
  else if (CHOOSE_BLUR==2)
  {
    blur_ori  = TextureColor.Sample(Sampler0, tex + float2(px,-py) * 0.5 * fOffsetBias).rgb;   // South East
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(-px,-py) * 0.5 * fOffsetBias).rgb;  // South West
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(px,py) * 0.5 * fOffsetBias).rgb;    // North East
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(-px,py) * 0.5 * fOffsetBias).rgb;   // North West

    blur_ori *= 0.25;  // ( /= 4) Divide by the number of texture fetches
  }

  // -- Pattern 3 -- An experimental 17 tap gaussian using 4+1 texture fetches.
  else if (CHOOSE_BLUR==3)
  {
    blur_ori  = TextureColor.Sample(Sampler0, tex + float2(0.4*px,-1.2*py)* fOffsetBias).rgb;    // South South East
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(-1.2*px,-0.4*py) * fOffsetBias).rgb;  // West South West
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(1.2*px,0.4*py) * fOffsetBias).rgb;    // East North East
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(-0.4*px,1.2*py) * fOffsetBias).rgb;   // North North West

    blur_ori *= 0.25;  // ( /= 4) Divide by the number of texture fetches
    sharp_strength_luma *= 0.51;
  }

  // -- Pattern 4 -- A 9 tap high pass (pyramid filter) using 4+1 texture fetches.
  else if (CHOOSE_BLUR==4)
  {
    blur_ori  = TextureColor.Sample(Sampler0, tex + float2(0.5 * px,-py * fOffsetBias)).rgb;   // South South East
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(fOffsetBias * -px,0.5 * -py)).rgb;  // West South West
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(fOffsetBias * px,0.5 * py)).rgb;    // East North East
    blur_ori += TextureColor.Sample(Sampler0, tex + float2(0.5 * -px,py * fOffsetBias)).rgb;   // North North West

    //blur_ori += (2 * ori); // Probably not needed. Only serves to lessen the effect.

    blur_ori /= 4.0;               // Divide by the number of texture fetches
    sharp_strength_luma *= 0.666;  // Adjust strength to aproximate the strength of pattern 2
  }


   /*-----------------------------------------------------------.
  /                            Sharpen                          /
  '-----------------------------------------------------------*/

  // Calculate the sharpening
  float3 sharp = ori - blur_ori;  //Subtracting the blurred image from the original image

    // Adjust strength of the sharpening and clamp it
    float4 sharp_strength_luma_clamp = float4(sharp_strength_luma * (0.5 / fSharpClamp),0.5); //Roll part of the clamp into the dot

    // sharp_luma = saturate((0.5 / fSharpClamp) * sharp_luma + 0.5); //scale up and clamp
    float sharp_luma = saturate(dot(float4(sharp,1.0), sharp_strength_luma_clamp)); //Calculate the luma, adjust the strength, scale up and clamp
    sharp_luma = (fSharpClamp * 2.0) * sharp_luma - fSharpClamp; //scale down

   // Apply depth based sharpening
    if (ENABLE_DEPTH==true)  sharp_luma = sharp_luma * (1.0f - linDepthFrom);

  // Combining the values to get the final sharpened pixel
  inColor = ori + sharp_luma;    // Add the sharpening to the the original.


   /*-----------------------------------------------------------.
  /                     Returning the output                    /
  '-----------------------------------------------------------*/

//  if (SHOW_EDGES==true)  inColor = abs(sharp * 4.0);
//  if (SHOW_EDGES==true)  inColor = saturate(0.5 + (sharp_luma * 4.0)).rrr;
  if (SHOW_EDGES==true)  inColor = sharp_luma;

  return saturate(inColor);
}