//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//        Contains ExtendedFX Sharpening Depth        //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Initial depth based Sharpening code         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the effect.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row1 <
  string UIName="A--------------------------A";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

//{

//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //

//21 tap reduced to 6 weights ( 41 pixels wide )
static const float sampleOffsets_3[6]   = { 0.0,         1.452313744966,      3.390210239952,      5.331472958797,      7.277552900121,      9.229394260785   };
static const float sampleWeights_3[6]   = { 0.142479385858,   0.244115579374,      0.131636577371,      0.043283482080,      0.008668409765,      0.001056258481   };


// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float4 ProcessGaussianH(float4 inColor, float2 inTexCoords)
{
  float sHeight       = ScreenSize.x * ScreenSize.w;
  float2 fvTexelSize  = float2(ScreenSize.y, 1.0 / sHeight);
  float px            = fvTexelSize.x;
  float py            = fvTexelSize.y;

    inColor = TextureColor.Sample(Sampler0, inTexCoords.xy) * sampleWeights_3[0];
    for(int i = 1; i < 6; ++i)
    {
      inColor += tex2D(SamplerColor2, inTexCoords.xy + float2(sampleOffsets_3[i]*px, 0.0)) * sampleWeights_3[i];
      inColor += tex2D(SamplerColor2, inTexCoords.xy - float2(sampleOffsets_3[i]*px, 0.0)) * sampleWeights_3[i];
    }

  inColor.w = 1.0f;
  return inColor;
}

float4 ProcessGaussianV(float4 inColor, float2 inTexCoords)
{
  float sHeight       = ScreenSize.x * ScreenSize.w;
  float2 fvTexelSize  = float2(ScreenSize.y, 1.0 / sHeight);
  float px            = fvTexelSize.x;
  float py            = fvTexelSize.y;

    inColor = TextureColor.Sample(Sampler0, inTexCoords.xy) * sampleWeights_3[0];
    for(int i = 1; i < 6; ++i)
    {
      color += tex2D(SamplerColor2, IN.txcoord.xy + float2(0.0, sampleOffsets_3[i]*py)) * sampleWeights_3[i];
      color += tex2D(SamplerColor2, IN.txcoord.xy - float2(0.0, sampleOffsets_3[i]*py)) * sampleWeights_3[i];
    }

  inColor.w = 1.0f;
  return inColor;
}

float4 ProcessLocalColor(float4 inColor, float2 inTexCoords)
{
  float4 scrcolor = 0.0;
  float4 color    = 0.0;
  float3 luma     = float3(0.2125, 0.7154, 0.0721);
  float  pixluma  = 0.0;
  float4 baseC    = tex2D(SamplerColor, IN.txcoord.xy);
  float4 blendC   = tex2D(SamplerColor2, IN.txcoord.xy);

   float3 base     = baseC.xyz;
   float3 blend    = blendC.xyz;

   scrcolor.xyz    = screen( base, blend );

   color.xyz       = lerp( base.xyz, scrcolor.xyz, gauss_bloom );

  color.w = 1.0f;
  return color;
}

  return saturate(inColor);
}