//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                  Contains Interlace                 //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// MTichenor: Author of Interlace                      //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the effect.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - DNSrSsEIFactor(WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight)
 * - DNEIFactor(NightExt, DayExt, NightInt, DayInt)                       *
 * - DNIFactor(NightExt, DayExt, Int)                                     *
 * - DNSrSsEIFactor(_c3, WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight) // ELE only!
 * - DNIDFactor(_c3, NightExt, DayExt, Interior, Dungeon) // ELE only!    *
 * - IDFactor(_c3, Interior, Dungeon)                     // ELE only!    *
 * - DunFactor(_c3, NightDayInt, Dungeon)                 // ELE only!    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row5 <
  string UIName="D--------------------------D";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

    bool ENABLE_HOR_INTERLACE <
        string UIName = "Enable Horizontal Interlace";
    > = {false};
    int fInterlaceHorizontalWidth <
        string UIName="Interlace: Horizontal Width";  string UIWidget="Spinner";  int UIMin=0;  int UIMax=10;
    > = {1.0};


    bool ENABLE_VER_INTERLACE <
        string UIName = "Enable Vertical Interlace";
    > = {false};
    int fInterlaceVerticalWidth <
        string UIName="Interlace: Vertical Width";    string UIWidget="Spinner";  int UIMin=0;  int UIMax=10;
    > = {1.0};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msInterHor(float4 inColor, float2 inTexCoords)
{
  if (ENABLE_HOR_INTERLACE==true)
  {
    float InterlaceHorizontalScalar = 0.5 / max(0.0, floor(fInterlaceHorizontalWidth));
    if(frac(inTexCoords.y * ScreenSize.x * ScreenSize.w * InterlaceHorizontalScalar) > 0.5) inColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
  }

  return inColor;
}

float3 msInterVer(float4 inColor, float2 inTexCoords)
{
  if (ENABLE_VER_INTERLACE==true)
  {
    float InterlaceVerticalScalar = 0.5 / max(0.0, floor(fInterlaceVerticalWidth));
    if(frac(inTexCoords.x * ScreenSize.x * InterlaceVerticalScalar) > 0.5) inColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
  }

  return inColor;
}