//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains Film Grain                 //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// MTichenor: Author of Film Grain                     //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the effect.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - DNSrSsEIFactor(WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight)
 * - DNEIFactor(NightExt, DayExt, NightInt, DayInt)                       *
 * - DNIFactor(NightExt, DayExt, Int)                                     *
 * - DNSrSsEIFactor(_c3, WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight) // ELE only!
 * - DNIDFactor(_c3, NightExt, DayExt, Interior, Dungeon) // ELE only!    *
 * - IDFactor(_c3, Interior, Dungeon)                     // ELE only!    *
 * - DunFactor(_c3, NightDayInt, Dungeon)                 // ELE only!    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row6 <
  string UIName="E--------------------------E";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

    bool ENABLE_GRAIN <
        string UIName = "Enable Grain";
    > = {false};

    float fGrainIntensity <
        string UIName="Grain: Intensity";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=0.5;  float UIStep=0.001;
    > = {0.035};
    float fGrainSaturation <
        string UIName="Grain: Saturation";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=0.5;  float UIStep=0.001;
    > = {0.0};
    float fGrainMotion <
        string UIName="Grain: Motion";      string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=0.2;  float UIStep=0.001;
    > = {0.2};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msGrain(float3 inColor, float2 inTexCoords)
{
  if (ENABLE_GRAIN==true)
  {
    float  GrainTimerSeed    = Timer.x * fGrainMotion;
    float2 GrainTexCoordSeed = inTexCoords.xy * 1.0f;

    float2 GrainSeed1  = GrainTexCoordSeed + float2( 0.0f, GrainTimerSeed );
    float2 GrainSeed2  = GrainTexCoordSeed + float2( GrainTimerSeed, 0.0f );
    float2 GrainSeed3  = GrainTexCoordSeed + float2( GrainTimerSeed, GrainTimerSeed );
    float  GrainNoise1 = random( GrainSeed1 );
    float  GrainNoise2 = random( GrainSeed2 );
    float  GrainNoise3 = random( GrainSeed3 );
    float  GrainNoise4 = ( GrainNoise1 + GrainNoise2 + GrainNoise3 ) * 0.333333333f;
    float3 GrainNoise  = float3( GrainNoise4, GrainNoise4, GrainNoise4 );
    float3 GrainColor  = float3( GrainNoise1, GrainNoise2, GrainNoise3 );

    inColor += ( lerp( GrainNoise, GrainColor, fGrainSaturation ) * fGrainIntensity ) - ( fGrainIntensity * 0.5f);
  }

  return inColor;
}