//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//               Contains Letterbox Bars               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// MTichenor: Author of Letterbox                      //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the effect.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - DNSrSsEIFactor(WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight)
 * - DNEIFactor(NightExt, DayExt, NightInt, DayInt)                       *
 * - DNIFactor(NightExt, DayExt, Int)                                     *
 * - DNSrSsEIFactor(_c3, WeatherAndTime.w, EXTSunrise, EXTDay, EXTSunset, EXTNight, INTSunrise, INTDay, INTSunset, INTNight) // ELE only!
 * - DNIDFactor(_c3, NightExt, DayExt, Interior, Dungeon) // ELE only!    *
 * - IDFactor(_c3, Interior, Dungeon)                     // ELE only!    *
 * - DunFactor(_c3, NightDayInt, Dungeon)                 // ELE only!    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row7 <
  string UIName="F--------------------------F";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};


    bool ENABLE_LETTERBOX <
        string UIName = "Enable Letterbox Bars";
    > = {false};
    float fLetterboxBarHeight <
        string UIName="Letterbox: Height in %";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=0.5;  float UIStep=0.001;
    > = {0.12};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

/// Simple Letterbox Bars
float4 msLetterbox(float4 inColor, float2 inTexCoords)
{
  if (ENABLE_LETTERBOX==true)
  {
    if (inTexCoords.y > 1.0f - fLetterboxBarHeight || inTexCoords.y  < fLetterboxBarHeight) inColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
  }

  return inColor;
}
