//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                ENBSeries effect file                //
//         visit http://enbdev.com for updates         //
//       Copyright (c) 2007-2015 Boris Vorontsov       //
//----------------------ENB PRESET---------------------//
//            - PHOTOREALISTIC COMMONWEALTH -          //
//                - ENBSeries v0.288 -                 //
//-----------------------CREDITS-----------------------//
// Boris: For ENBSeries and his knowledge and codes    //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//




//++++++++++++++++++++++++++++++++++++++++++++++++++++
// INTERNAL PARAMETERS BEGINS HERE, CAN BE MODIFIED
//++++++++++++++++++++++++++++++++++++++++++++++++++++

  bool ENABLE_HLSENSE <
    string UIName = "Enable Hightlight Sensitivity";
  > = {false};

  int CHOOSE_OUTPUT <
    string UIName = "Choose Gray Output";  string UIWidget="Spinner";  int UIMin=1;  int UIMax=3;
  > = {1};
  float3 fRGB_Out <
    string UIName = "Alter Gray RGB Output nr2";  string UIWidget="Color";
  > = {0.3333, 0.3333, 0.3333};


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// EXTERNAL PARAMETERS BEGINS HERE, SHOULD NOT BE MODIFIED UNLESS YOU KNOW WHAT YOU ARE DOING
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4 Timer;            /// x = generic timer in range 0..1, period of 16777216 ms (4.6 hours), y = average fps, w = frame time elapsed (in seconds)
float4 ScreenSize;       /// x = Width, y = 1/Width, z = aspect, w = 1/aspect, aspect is Width/Height

// External ENB debugging paramaters
float4 tempF1;     /// 0,1,2,3  // Keyboard controlled temporary variables.
float4 tempF2;     /// 5,6,7,8  // Press and hold key 1,2,3...8 together with PageUp or PageDown to modify.
float4 tempF3;     /// 9,0
float4 tempInfo1;  /// xy = cursor position in range 0..1 of screen, z = is shader editor window active, w = mouse buttons with values 0..7
/// tempInfo1 assigned mouse button values
///    0 = none
///    1 = left
///    2 = right
///    3 = left+right
///    4 = middle
///    5 = left+middle
///    6 = right+middle
///    7 = left+right+middle (or rather cat is sitting on your mouse)

float4 AdaptationParameters;  // x = AdaptationMin, y = AdaptationMax, z = AdaptationSensitivity, w = AdaptationTime multiplied by time elapsed


// TEXTURES
Texture2D TextureCurrent;
Texture2D TexturePrevious;

// SAMPLERS
SamplerState Sampler0
{
  Filter=MIN_MAG_MIP_POINT;  AddressU=Clamp;  AddressV=Clamp;
};
SamplerState Sampler1
{
  Filter=MIN_MAG_MIP_LINEAR;  AddressU=Clamp;  AddressV=Clamp;
};


// DATA STRUCTURE
struct VS_INPUT_POST
{
  float3 pos     : POSITION;
  float2 txcoord : TEXCOORD0;
};
struct VS_OUTPUT_POST
{
  float4 pos      : SV_POSITION;
  float2 txcoord0 : TEXCOORD0;
};


// HELPER FUNCTIONS
/// Include the additional shader files containing all helper functions and constants
   #include "/Modular Shaders/msHelpers.fxh"                                /// Required for all files below!


// VERTEX SHADER
VS_OUTPUT_POST VS_Quad(VS_INPUT_POST IN, uniform float sizeX, uniform float sizeY)
{
  VS_OUTPUT_POST OUT;

    float4 pos;
    pos.xyz         = IN.pos.xyz;
    pos.w           = 1.0;
    OUT.pos         = pos;
    float2 offset;
    offset.x        = sizeX;
    offset.y        = sizeY;
    OUT.txcoord0.xy = IN.txcoord.xy + offset.xy;

  return OUT;
}



// PIXEL SHADER
///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/// output size is 16*16
/// TextureCurrent size is 256*256, it's internally downscaled from full screen
/// input texture is R16G16B16A16 or R11G11B10 float format (alpha ignored)
/// output texture is R32 float format (red channel only)
///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
float4 PS_Downsample(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

  /// Downsample 256*256 to 16*16
  /// More complex blurring methods affect result if sensitivity uncommented
    float2 pos;
    float2 coord;
    float4 curr          = 0.0;
    float4 currmax       = 0.0;
    const float scale    = 1.0 / 16.0;
    const float step     = 1.0 / 16.0;
    const float halfstep = 0.5 / 16.0;
    pos.x                = -0.5 + halfstep;
    for (int x=0; x<16; x++)
    {
      pos.y = -0.5 + halfstep;
      for (int y=0; y<16; y++)
      {
        coord           = pos.xy * scale;
        float4 tempcurr = TextureCurrent.Sample(Sampler0, IN.txcoord0.xy + coord.xy);
        currmax         = max(currmax, tempcurr);
        curr           += tempcurr;
        pos.y          += step;
      }
      pos.x += step;
    }
    curr *= 1.0 / (16.0 * 16.0);

    res = curr;

  // Adjust sensitivity to small bright areas on the screen
  // Warning! Uncommenting the next line increases sensitivity a lot
  if (ENABLE_HLSENSE==true)    res = lerp(curr, currmax, AdaptationParameters.z); //AdaptationSensitivity

  ///TODO modify this math to your taste, for example lower intensity for blue colors
  ///gray output
  ///v1
  if (CHOOSE_OUTPUT==1)  res=AvgLuma(res.xyz).y;
  ///v2
  if (CHOOSE_OUTPUT==2)  res=dot(res.xyz, float3(fRGB_Out.x, fRGB_Out.y, fRGB_Out.z));
  ///v3
  if (CHOOSE_OUTPUT==3)  res=AvgLuma(res.xyz).w;


  res.w=1.0;
  return res;
}


///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/// output size is 1*1
/// TexturePrevious size is 1*1
/// TextureCurrent size is 16*16
/// output and input textures are R32 float format (red channel only)
///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
float4 PS_Adaptation(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

  float prev = TexturePrevious.Sample(Sampler0, IN.txcoord0.xy).x;

  // Downsample 16*16 to 1*1
    float2 pos;
    float curr           = 0.0;
    float currmax        = 0.0;
    const float step     = 1.0/16.0;
    const float halfstep = 0.5/16.0;
    pos.x                = halfstep;

    for (int x=0; x<16; x++)
    {
      pos.y = halfstep;
      for (int y=0; y<16; y++)
        {
          float tempcurr = TextureCurrent.Sample(Sampler0, IN.txcoord0.xy + pos.xy).x;
          currmax       = max(currmax, tempcurr);
          curr         += tempcurr;
          pos.y        += step;
        }
      pos.x += step;
    }
    curr *= 1.0/(16.0*16.0);

  // Adjust sensitivity to small bright areas on the screen
    curr = lerp(curr, currmax, AdaptationParameters.z); //AdaptationSensitivity

  // Smooth by time
    res = lerp(prev, curr, AdaptationParameters.w); //AdaptationTime with elapsed time

  // Clamp to avoid bugs in post process shader, which have much lower floating point precision
    res = max(res, 0.001);
    res = min(res, 16384.0);

  // Limit value if ForceMinMaxValues=true
    float valmax;
    float valcut;
    valmax = max(res.x, max(res.y, res.z));
    valcut = max(valmax, AdaptationParameters.x); //AdaptationMin
    valcut = min(valcut, AdaptationParameters.y); //AdaptationMax
    res   *= valcut/(valmax + 0.000000001f);

  res.w=1.0;
  return res;
}


// TECHNIQUES
technique11 Downsample  /// First pass for downscaling and computing sensitivity
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad(0.0, 0.0)));
    SetPixelShader(CompileShader(ps_5_0, PS_Downsample()));
  }
}


technique11 Draw //<string UIName="ENBAdaptation";>  /// Last pass for mixing everything
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad(0.0, 0.0)));
    SetPixelShader(CompileShader(ps_5_0, PS_Adaptation()));
  }
}