//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                ENBSeries effect file                //
//         visit http://enbdev.com for updates         //
//       Copyright (c) 2007-2015 Boris Vorontsov       //
//----------------------ENB PRESET---------------------//
//          - PhotoRealistic Commonwealth LITE -       //
//                - ENBSeries v0.288 -                 //
//-----------------------CREDITS-----------------------//
// Boris: For ENBSeries and his knowledge and codes    //
// JawZ: Author and developer of this file             //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//




//++++++++++++++++++++++++++++++++++++++++++++++++++++
// INTERNAL PARAMETERS BEGINS HERE, CAN BE MODIFIED
//++++++++++++++++++++++++++++++++++++++++++++++++++++

float INFO <
  string UIName="Activate effects with TECHNQIUE tab above";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

#define ACTIVATE_SHARPENING    3    /// 1, 2, 3, 4. Choose one of the given numbers to change Sharpening method.

/// SHARPENING : 1=ENB Default, 2=SharpFX, 3=SharpFX Depth, 4=SweetFX Lumasharpen.


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// EXTERNAL PARAMETERS BEGINS HERE, SHOULD NOT BE MODIFIED UNLESS YOU KNOW WHAT YOU ARE DOING
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4 Timer;       /// x = generic timer in range 0..1, period of 16777216 ms (4.6 hours), y = average fps, w = frame time elapsed (in seconds)
float4 ScreenSize;  /// x = Width, y = 1/Width, z = aspect, w = 1/aspect, aspect is Width/Height

// External ENB debugging paramaters
float4 tempF1;     /// 0,1,2,3  // Keyboard controlled temporary variables.
float4 tempF2;     /// 5,6,7,8  // Press and hold key 1,2,3...8 together with PageUp or PageDown to modify.
float4 tempF3;     /// 9,0
float4 tempInfo1;  /// xy = cursor position in range 0..1 of screen, z = is shader editor window active, w = mouse buttons with values 0..7
/// tempInfo1 assigned mouse button values
///    0 = none
///    1 = left
///    2 = right
///    3 = left+right
///    4 = middle
///    5 = left+middle
///    6 = right+middle
///    7 = left+right+middle (or rather cat is sitting on your mouse)


// TEXTURES
Texture2D TextureOriginal;  /// LDR color
Texture2D TextureColor;     /// LDR color which is output of previous technique
Texture2D TextureDepth;     /// scene depth

/// Temporary textures which can be set as render target for techniques via annotations like <string RenderTarget="RenderTargetRGBA32";>
Texture2D RenderTargetRGBA32;   // R8G8B8A8 32 bit ldr format
Texture2D RenderTargetRGBA64;   // R16B16G16A16 64 bit ldr format
Texture2D RenderTargetRGBA64F;  // R16B16G16A16F 64 bit hdr format
Texture2D RenderTargetR16F;     // R16F 16 bit hdr format with red channel only
Texture2D RenderTargetR32F;     // R32F 32 bit hdr format with red channel only
Texture2D RenderTargetRGB32F;   // 32 bit hdr format without alpha


// SAMPLERS
SamplerState Sampler0
{
  Filter=MIN_MAG_MIP_POINT;  AddressU=Clamp;  AddressV=Clamp;  // MIN_MAG_MIP_LINEAR;
};
SamplerState Sampler1
{
  Filter=MIN_MAG_MIP_LINEAR;  AddressU=Clamp;  AddressV=Clamp;
};


// DATA STRUCTURE
struct VS_INPUT_POST
{
  float3 pos     : POSITION;
  float2 txcoord : TEXCOORD0;
};
struct VS_OUTPUT_POST
{
  float4 pos      : SV_POSITION;
  float2 txcoord0 : TEXCOORD0;
};


// HELPER FUNCTIONS
/// Include the additional shader files containing all helper functions and constants
   #include "/Modular Shaders/msHelpers.fxh"  /// Required for all files below!
//   #include "/Modular Shaders/msDrawText.fxh"
#if ACTIVATE_SHARPENING == 1
   #include "/Modular Shaders/enbeffectpostpass/enbBlurSharp.fxh"
#endif
#if ACTIVATE_SHARPENING == 2
   #include "/Modular Shaders/enbeffectpostpass/10_Sharpen.fxh"
#endif
#if ACTIVATE_SHARPENING == 3
   #include "/Modular Shaders/enbeffectpostpass/11_SharpenDepth.fxh"
#endif
#if ACTIVATE_SHARPENING == 4
   #include "/Modular Shaders/enbeffectpostpass/12_SFXSharpenDepth.fxh"
#endif
   #include "/Modular Shaders/enbeffectpostpass/20_Vignette.fxh"
   #include "/Modular Shaders/enbeffectpostpass/21_Dither.fxh"
//   #include "/Modular Shaders/enbeffectpostpass/22_Interlace.fxh"
   #include "/Modular Shaders/enbeffectpostpass/23_Grain.fxh"
   #include "/Modular Shaders/enbeffectpostpass/24_Letterbox.fxh"

//21 tap reduced to 6 weights ( 41 pixels wide )
static const float sampleOffsets_3[6]   = { 0.0,         1.452313744966,      3.390210239952,      5.331472958797,      7.277552900121,      9.229394260785   };
static const float sampleWeights_3[6]   = { 0.142479385858,   0.244115579374,      0.131636577371,      0.043283482080,      0.008668409765,      0.001056258481   };

float3 screen(float3 a, float3 b)
{
   return (1.0f - (1.0f - a) * (1.0f - b));
}


// VERTEX SHADER
VS_OUTPUT_POST VS_PostProcess(VS_INPUT_POST IN)
{
  VS_OUTPUT_POST OUT;

    float4 pos;
    pos.xyz=IN.pos.xyz;
    pos.w=1.0;
    OUT.pos=pos;
    OUT.txcoord0.xy=IN.txcoord.xy;

  return OUT;
}


// PIXEL SHADERS
/// ENBSeries default Blurring and Sharpening
#if ACTIVATE_SHARPENING == 1
float4 PS_Blur(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = enbBlur(res, IN.txcoord0.xy);  /// For use with enbBlurSharp.fxh

  res.w = 1.0;
  return res;
}

float4 PS_Sharp(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = enbSharpen(res, IN.txcoord0.xy);  /// For use with enbBlurSharp.fxh

  res.w = 1.0;
  return res;
}
#endif

/// SharpenFX
#if ACTIVATE_SHARPENING >= 2 && ACTIVATE_SHARPENING <= 3
float4 PS_BlurH(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = msGaussianH(res, IN.txcoord0.xy);  /// For use with 10_Sharpen.fxh and 11_SharpenDepth.fxh

  res.w = 1.0;
  return res;
}

float4 PS_BlurV(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = msGaussianV(res, IN.txcoord0.xy);  /// For use with 10_Sharpen.fxh and 11_SharpenDepth.fxh

  res.w = 1.0;
  return res;
}

float4 PS_Edges(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = msEdges(res, IN.txcoord0.xy);  /// For use with 10_Sharpen.fxh and 11_SharpenDepth.fxh

  res.w = 1.0;
  return res;
}

float4 PS_Sharpen1(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = msSharpen1(res, IN.txcoord0.xy);  /// For use with 10_Sharpen.fxh and 11_SharpenDepth.fxh

  res.w = 1.0;
  return res;
}

float4 PS_Sharpen2(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = msSharpen2(res, IN.txcoord0.xy);  /// For use with 10_Sharpen.fxh and 11_SharpenDepth.fxh

  res.w = 1.0;
  return res;
}
#endif

/// SFX Lumasharpen
#if ACTIVATE_SHARPENING == 4
float4 PS_SFXSharpen(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

    res = TextureColor.Sample(Sampler0, IN.txcoord0.xy);

    res.xyz = LumaSharpenPass(res.xyz, IN.txcoord0.xy);  /// For use with 10_Sharpen.fxh and 11_SharpenDepth.fxh

  res.w = 1.0;
  return res;
}
#endif

/// Post Effects
float4 PS_PostFX(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;
    res = TextureColor.Sample(Sampler0, IN.txcoord0.xy);

    res     = msVignette(res, IN.txcoord0.xy);    /// Requires 20_Vignette.fxh active

    res.rgb = msDither(res.rgb, IN.txcoord0.xy);  /// Requires 21_Dither.fxh active

//    res.rgb = msInterHor(res, IN.txcoord0.xy);    /// Requires 22_Interlace.fxh active
//    res.rgb = msInterVer(res, IN.txcoord0.xy);    /// Requires 22_Interlace.fxh active

    res.rgb = msGrain(res.rgb, IN.txcoord0.xy);   /// Requires 23_Grain.fxh active

    res     = msLetterbox(res, IN.txcoord0.xy);   /// Requires 24_Letterbox.fxh active

  res.w = 1.0;
//    float2 Text_0[Text_SIZE] = string( D, e, m, o, T, e, x, t, Space, b);   //Test
//    float2 Text_1[Text_SIZE] = { Char_y, Char_Colon, Char_K, Char_i, Char_n, Char_g, Char_E, Char_r, Char_i,     Char_c};   //Test
//    res += DrawText_String(float2(0                                 , 800), 32, 5, IN.txcoord0.xy,  Text_0);
//    res += DrawText_String(float2(DrawText_Width(10, 32, 5), 800), 32, 5, IN.txcoord0.xy,  Text_1);
  return res;
}


// TECHNIQUES
///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/// Techniques are drawed one after another and they use as result of
/// previous as input color. Number of techniques is limited to 255.
/// If UIName specified, then it's base technique which may have extra
/// techniques with indexing
///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/// Blur and sharpening example, applies two blur passes and one sharpening pass
#if ACTIVATE_SHARPENING == 1
technique11 BlurSharp <string UIName="BlurSharp";>  /// First  Blur Pass
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_Blur()));
  }
}

technique11 BlurSharp1  /// Second Blur Pass
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_Blur()));
  }
}

technique11 BlurSharp2  /// First Sharpening Pass
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_Sharp()));
  }
}

technique11 BlurSharp3  /// Post effects
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_PostFX()));
  }
}
#endif

#if ACTIVATE_SHARPENING >= 2 && ACTIVATE_SHARPENING <= 3
// SharpenFX
technique11 SharpenFX <string UIName="SharpenFX";>  /// Horizontal Gaussian blur
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_BlurH()));
  }
}

technique11 SharpenFX1  /// Vertical Gaussian blur
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_BlurV()));
  }
}

technique11 SharpenFX2  /// Edge detection
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_Edges()));
  }
}

technique11 SharpenFX3  /// First Sharpening Pass
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_Sharpen1()));
  }
}

technique11 SharpenFX4  /// Second Sharpening Pass
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_Sharpen2()));
  }
}

technique11 SharpenFX5  /// Post effects
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_PostFX()));
  }
}
#endif

#if ACTIVATE_SHARPENING == 4
technique11 SFXLumaSharp <string UIName="SFX Luma Sharpen";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_SFXSharpen()));
  }
}

technique11 SFXLumaSharp1  /// Post effects
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_PostProcess()));
    SetPixelShader(CompileShader(ps_5_0, PS_PostFX()));
  }
}
#endif
#include "enbsweet.fx"