//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains ENB Palette                //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// kingeric1992: 3D LUT code development               //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the elepHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
#ifdef E_CC_PROCEDURAL
  bool ENABLE_3DLUT <
    string UIName = "Enable 3D LUT";
  > = {true};
#endif

// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //

Texture2D TextureLut <        /// 3D LUT texture map
  string UIName = "3DLut";  string ResourceName = "enbseries/Modular Shaders/Film_FujiProvia100F.png";
>;

SamplerState SamplerLut {
   Filter=MIN_MAG_MIP_LINEAR;  AddressU=Clamp;  AddressV=Clamp;
};


// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 ms3DLUT(float3 inColor)
{
#ifdef E_CC_PROCEDURAL
  if (ENABLE_3DLUT==true)
  {
    float2 CLut_pSize = float2(0.000244140625, 0.015625);  // 1 / float2(256, 16);
    inColor.rgb       = saturate(inColor.rgb);
    inColor.b        *= 63;
    float4 CLut_UV    = 0;
    CLut_UV.w         = floor(inColor.b);
    CLut_UV.xy        = inColor.rg * 63 * CLut_pSize + 0.5 * CLut_pSize ;
    CLut_UV.x        += CLut_UV.w * CLut_pSize.y;
    inColor.rgb       = lerp(TextureLut.SampleLevel(SamplerLut, CLut_UV.xyzz, 0).rgb, TextureLut.SampleLevel(SamplerLut, CLut_UV.xyzz + float4(CLut_pSize.y, 0, 0, 0), 0).rgb, inColor.b - CLut_UV.w);
  }
#endif
  return saturate(inColor.rgb);
}